package com.rits.orm

/**
 * @author kostantinos.kougios
 *
 * 25 Jul 2011
 */
class TypeRegistry {
	class TypeInfo[T](val tpe: Type[T])

	private var typesBuilder = Map.newBuilder[Class[_], TypeInfo[_]]
	private var types: Map[Class[_], TypeInfo[_]] = _

	/**
	 * registration of entities
	 */
	def registerType[T](tpe: Type[T]): Unit =
		{
			if (types != null) throw new IllegalStateException("already initialized, cant register types after initialization")

			val ti = new TypeInfo[T](tpe)
			typesBuilder += tpe.clz -> ti
		}

	private def checkInit(): Unit =
		{
			if (types == null) throw new IllegalStateException("Not yet initialized. Please call init method after registering all types")
		}
	/**
	 * call this after registering all types
	 */
	def init: Unit =
		{
			if (types != null) throw new IllegalStateException("already initialized. Please init only once after registering all types")
			types = typesBuilder.result
			typesBuilder = null
		}

	/**
	 * utility methods
	 */
	protected[orm] def typeInfo[T](clz: Class[T]): TypeInfo[T] =
		{
			val ti = types(clz)
			ti.asInstanceOf[TypeInfo[T]]
		}

	protected[orm] def typeInfo[T](o: T): TypeInfo[T] =
		{
			checkInit

			// there is a very weird compilation error from maven
			// if I just do val clz = o.getClass.asInstanceOf[Class[T]]
			val clz = o.asInstanceOf[Object].getClass.asInstanceOf[Class[T]]
			o match {
				case p: Persisted => types.get(clz.getSuperclass).get.asInstanceOf[TypeInfo[T]]
				case _ => typeInfo(clz)
			}
		}
}

object TypeRegistry {
	/**
	 * creates a TypeRegistry, registers all types and initializes the TypeRegistry.
	 */
	def apply(types: Type[_]*): TypeRegistry =
		{
			val registry = new TypeRegistry
			types.foreach(registry.registerType(_))
			registry.init
			registry
		}
}