package com.rits.orm
import scala.collection.mutable.Buffer
import java.util.Calendar
import org.joda.time.DateTime

/**
 * @author kostantinos.kougios
 *
 * 16 Jul 2011
 */
class ValuesMap(typeManager: TypeManager, m: Map[String, Any]) {
	protected[orm] def apply[T](column: String): T = typeManager.deepClone(m(column).asInstanceOf[T])

	/**
	 * the following methods just cast the value
	 * and are provided for type-safe convinience
	 */
	def int(column: String): Int = apply(column)
	def float(column: String): Float = apply(column)
	def double(column: String): Double = apply(column)
	def string(column: String): String = apply(column)
	def calendar(column: String): Calendar = apply(column)
	def datetime(column: String): DateTime = apply(column)

	/**
	 * the following methods do a conversion
	 */
	def traversable[T](column: String): Traversable[T] = apply(column).asInstanceOf[Traversable[T]]
	def list[T](column: String): List[T] = apply(column).asInstanceOf[Traversable[T]].toList
	def set[T](column: String): Set[T] = apply(column).asInstanceOf[Traversable[T]].toSet
	def seq[T](column: String): Seq[T] = apply(column).asInstanceOf[Traversable[T]].toSeq
	def indexedSeq[T](column: String): Seq[T] = apply(column).asInstanceOf[Traversable[T]].toIndexedSeq
	def buffer[T](column: String): Buffer[T] = apply(column).asInstanceOf[Traversable[T]].toBuffer

	override def toString = m.toString

	protected[orm] def toMutableMap: scala.collection.mutable.Map[String, Any] = new scala.collection.mutable.HashMap() ++ m

	protected[orm] def toListOfColumnAndValueTuple(columns: List[ColumnBase]) = columns.map(c => (c, m(c.columnName)))
}

object ValuesMap {
	protected[orm] def fromEntity[T](typeManager: TypeManager, tpe: Type[T], o: T): ValuesMap =
		{
			val m = tpe.table.toColumnAliasAndValueMap(tpe.table.columnsWithoutAutoGenerated, o).map(e => (e._1, typeManager.deepClone(e._2)))
			new ValuesMap(typeManager, m)
		}
	protected[orm] def fromMutableMap(typeManager: TypeManager, m: scala.collection.mutable.Map[String, Any]): ValuesMap =
		{
			new ValuesMap(typeManager, Map() ++ m)
		}
}